{% extends "media/base_photos.html" %}


{% block title %}{{ object.title }}{% endblock %}

{% block content_title %}<h2>{{ object.title }}</h2>{% endblock %}

{% block content %}
  <p class="date">{{ object.uploaded|date:"j F Y" }}</p>

  <div class="body">
    <img src="{{ object.photo.url }}" style="width: 500px;" />
  </div>

  <h3>Related to:</h3>
  <ul>
  {% for obj in object.related.all.generic_objects %}
    <li><a href="{{ obj.get_absolute_url }}">{{ obj }}</a></li>
  {% empty %}
    <li>Nothing here</li>
  {% endfor %}
  </ul>
{% endblock %}
