.djn-group .djn-group-nested {
  float: none;
  width: auto;
  margin: 0 10px;
  background: transparent;
}

.djn-group-nested.grp-stacked h2.djn-collapse-handler,
.djn-group-nested.grp-stacked > .grp-tools {
  display: none;
}

.djn-group-nested {
  border-color: transparent;
}

.grp-tools span.delete {
  cursor: auto !important;
}

.djn-group-nested .djn-items .inline-related {
  border: 1px solid transparent;
  border-radius: 4px;
}

#grp-content .djn-group-nested .djn-items .inline-related {
  margin-bottom: 5px;
  border: 1px solid #a7a7a7;
}

#grp-content .djn-group-nested .djn-items .inline-related.djn-item-dragging {
  border: 0;
}

.djn-group-nested .djn-items .inline-related:first-child {
  margin-top: 0;
}

.djn-group-nested .djn-items .inline-related.last-related {
  margin-bottom: 0;
}

.djn-group-nested div.items .module:first-child {
  margin-top: 0 !important;
}

.nested-placeholder, .djn-group .ui-sortable-placeholder {
  margin-bottom: 5px;
  background: #9f9f9f !important;
}

.djn-group .ui-nestedsortable-error {
  background: #9f6464 !important;
}

.djn-items {
  position: relative;
  min-height: 0;
  overflow: visible;
}

.djn-item {
  overflow: visible;
}

.djn-item.djn-no-drag:first-child {
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
  z-index: -1;
  height: 19px;
}

.djn-item.djn-no-drag:first-child + .djn-item.ui-sortable-helper,
.djn-item.djn-no-drag:first-child + .djn-item-dragging {
  margin-top: 0;
}

.djn-item-dragging {
  height: 0;
  padding: 0;
  margin: 0;
  overflow: hidden;
  border: 0;
}

.djn-tbody.djn-item-dragging {
  display: none !important;
}

.djn-tbody.ui-sortable-placeholder td {
  background: #fbfad0;
}

.djn-collapse-handler-verbose-name {
  display: inline;
}

#grp-content .grp-tabular .grp-table .grp-tbody .grp-th,
#grp-content .grp-tabular .grp-table .grp-tbody .grp-td {
  vertical-align: top;
  overflow: visible;
}

#grp-content .grp-tabular .grp-table .grp-tbody .grp-tr > td.original:first-child {
  width: 0;
  padding: 0;
  border: 0;
  background: #eee;
}

#grp-content .grp-tabular .grp-table .grp-tbody .grp-tr.djn-has-inlines .grp-td {
  border-bottom: 0 !important;
}

#grp-content .grp-tabular .grp-table .grp-thead .grp-th {
  border-radius: 0;
  border-top: 0;
  border-bottom: 0;
  line-height: 16px;
  color: #aaa;
  font-weight: bold;
}

#grp-content table.djn-table thead > tr > th {
  font-size: 11px;
  line-height: inherit;
}

#grp-content .grp-tabular .grp-table.djn-table .grp-thead > .grp-tr > .grp-th {
  padding-top: 1px;
  padding-bottom: 1px;
}

#grp-content .grp-tabular .grp-table.djn-table .grp-thead > .grp-tr > .grp-th:last-of-type {
  border-right: 0;
}

#grp-content .grp-tabular .grp-table.djn-table .grp-tbody > .grp-tr > .grp-td:first-of-type {
  border-left: 1px solid #d4d4d4 !important;
}

table.djn-table.grp-table td div.grp-readonly,
table.djn-table.grp-table th div.grp-readonly {
  margin: 0 !important;
}

.grp-tabular.djn-tabular td.grp-td ul.errorlist {
  margin: 0 !important;
}

table.djn-table.grp-table td div.grp-readonly:empty,
table.djn-table.grp-table th div.grp-readonly:empty {
  margin-bottom: -5px !important;
}

table.djn-table.grp-table td > input[type="checkbox"],
table.djn-table.grp-table td > input[type="radio"],
table.djn-table.grp-table th > input[type="checkbox"],
table.djn-table.grp-table th > input[type="radio"] {
  margin: 3px 0.5ex !important;
  margin: revert !important;
}

table.djn-table.grp-table td > textarea,
table.djn-table.grp-table th > textarea {
  margin: 0 !important;
}

table.djn-table.grp-table td > input[type="text"],
table.djn-table.grp-table td > input[type="password"],
table.djn-table.grp-table td > input[type="url"],
table.djn-table.grp-table td > input[type="email"],
table.djn-table.grp-table td > input[type="number"],
table.djn-table.grp-table td > input[type="button"],
table.djn-table.grp-table td > select,
table.djn-table.grp-table td p input[type="text"],
table.djn-table.grp-table td p input[type="url"],
table.djn-table.grp-table td p input[type="email"],
table.djn-table.grp-table td p input[type="number"],
table.djn-table.grp-table td p > input[type="button"],
table.djn-table.grp-table th > input[type="text"],
table.djn-table.grp-table th > input[type="password"],
table.djn-table.grp-table th > input[type="url"],
table.djn-table.grp-table th > input[type="email"],
table.djn-table.grp-table th > input[type="number"],
table.djn-table.grp-table th > input[type="button"],
table.djn-table.grp-table th > select,
table.djn-table.grp-table th p input[type="text"],
table.djn-table.grp-table th p input[type="url"],
table.djn-table.grp-table th p input[type="email"],
table.djn-table.grp-table th p input[type="number"],
table.djn-table.grp-table th p > input[type="button"] {
  vertical-align: middle;
  margin-top: 0 !important;
  margin-bottom: 0 !important;
}

.djn-empty-form, .djn-empty-form * {
  display: none !important;
}

#content.colM .inline-group .tabular .ui-sortable-placeholder tr.has_original td {
  padding: 1px;
}

#content.colM .inline-group.djn-group ul.tools {
  height: 0;
}

#content.colM .djn-item.module {
  margin-bottom: 0;
}

#content.colM tr.djn-has-inlines td {
  border-bottom: 1px solid #fff;
}

#content.colM td.original {
  width: 0;
  padding: 2px 0 0 0;
}

#content.colM td.original.is-sortable {
  position: relative;
  width: 15px;
}

#content.colM td.original.is-sortable .djn-drag-handler {
  position: absolute;
  top: 4px;
  left: 0;
  display: block;
  width: 10px;
  height: 20px;
  margin: 5px;
  cursor: move;
  background: url("data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAyCAYAAABcfPsmAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAE5JREFUeNrs0zEKACAMBEEDST6X9+Z92lpYHNgI7naBK9KMdfcch6rK9lvdeWYOJXXnESEN1d2HH9J9hhSkEFKQQkhBClKQghSkPNYSYADFZiuygfao+AAAAABJRU5ErkJggg==") no-repeat top left;
  background-size: 10px 25px;
  cursor: -webkit-grab;
  cursor: grab;
  cursor: -moz- -webkit-grab;
  cursor: -moz- grab;
  cursor: -webkit- -webkit-grab;
  cursor: -webkit- grab;
}

#content.colM td.original.is-sortable .djn-drag-handler:active {
  cursor: -webkit-grabbing;
  cursor: grabbing;
  cursor: -moz- -webkit-grabbing;
  cursor: -moz- grabbing;
  cursor: -webkit- -webkit-grabbing;
  cursor: -webkit- grabbing;
}

#content.colM td.original.is-sortable p + .djn-drag-handler {
  top: 20px;
}

#content.colM td.original.is-sortable p {
  top: 0;
  left: 19px;
}

#content.colM fieldset.has-inlines > .djn-form-row-last {
  border-bottom: 0;
}

#suit-center .form-horizontal .djn-no-drag + .inline-related h3 {
  margin-top: 0;
}

/*# sourceMappingURL=nested_admin.css.map */
